@@ -47,7 +47,7 @@ AbstractBackend::Capabilities Motis2Backend::capabilities() const
 bool Motis2Backend::needsLocationQuery(const Location &loc, AbstractBackend::QueryType type) const
 {
     if (type == QueryType::Journey) {
-        return !loc.hasCoordinate() && !loc.hasIdentifier(m_locationIdentifierType);
+        return (!loc.hasCoordinate() || !m_supportsStreetRouting) && !loc.hasIdentifier(m_locationIdentifierType);
     }
     return !loc.hasIdentifier(m_locationIdentifierType);
 }
@@ -108,6 +108,13 @@ bool Motis2Backend::queryLocation(const LocationRequest &req, LocationReply *rep
     } else {
         query.addQueryItem(u"text"_s, req.name());
         query.addQueryItem(u"language"_s, preferredLanguage());
+        if (req.types() == Location::Stop) {
+            query.addQueryItem(u"type"_s, u"STOP"_s);
+        }
+        if (req.viewbox().isValid()) {
+            query.addQueryItem(u"place"_s, QString::number(req.viewbox().center().y()) + ','_L1 + QString::number(req.viewbox().center().x()));
+        }
+
         netReply = makeRequest(req, reply, "geocode"_L1, query, nam);
     }
 
@@ -152,16 +159,25 @@ bool Motis2Backend::queryStopover(const StopoverRequest &req, StopoverReply *rep
     query.addQueryItem(u"radius"_s, u"200"_s);
 
     auto netReply = makeRequest(req, reply, "stoptimes"_L1, query, nam);
-    QObject::connect(netReply, &QNetworkReply::finished, reply, [this, netReply, reply]() {
+    QObject::connect(netReply, &QNetworkReply::finished, reply, [this, netReply, reply, req]() {
         netReply->deleteLater();
         const auto data = netReply->readAll();
         logReply(reply, netReply, data);
 
         Motis2Parser p(m_locationIdentifierType);
-        auto result = p.parseStopTimes(data);
+        auto result = p.parseStopTimes(data, reply->request().mode() == StopoverRequest::QueryArrival);
         if (netReply->error() == QNetworkReply::NoError) {
             setNextRequestContext(reply, p.m_nextPageCursor);
             setPreviousRequestContext(reply, p.m_previousPageCursor);
+
+            // MOTIS has no support for doing mode filtering on the server
+            if (!req.lineModes().empty()) {
+                auto it = std::remove_if(result.begin(), result.end(), [req](const auto &s) {
+                    return !std::binary_search(req.lineModes().begin(), req.lineModes().end(), s.route().line().mode());
+                });
+                result.erase(it, result.end());
+            }
+
             addResult(reply, this, std::move(result));
         } else {
             addError(reply, Reply::NetworkError, netReply->errorString() + ' '_L1 + QString::fromUtf8(data));
@@ -171,12 +187,12 @@ bool Motis2Backend::queryStopover(const StopoverRequest &req, StopoverReply *rep
     return true;
 }
 
-[[nodiscard]] static QString encodeLocation(const Location &loc, const QString &locationIdentifierType)
+QString Motis2Backend::encodeLocation(const Location &loc) const
 {
-    if (loc.hasCoordinate()) {
+    if (loc.hasCoordinate() && m_supportsStreetRouting) {
         return QString::number(loc.latitude()) + ','_L1 + QString::number(loc.longitude()) + ','_L1 + QString::number(loc.hasFloorLevel() ? loc.floorLevel() : 0);
     }
-    return loc.identifier(locationIdentifierType);
+    return loc.identifier(m_locationIdentifierType);
 }
 
 struct {
@@ -249,9 +265,10 @@ static void mapIndividualTransportFormFactors(const std::vector<IndividualTransp
 bool Motis2Backend::queryJourney(const JourneyRequest &req, JourneyReply *reply, QNetworkAccessManager *nam) const
 {
     QUrlQuery query;
-    query.addQueryItem(u"fromPlace"_s, encodeLocation(req.from(), m_locationIdentifierType));
-    query.addQueryItem(u"toPlace"_s, encodeLocation(req.to(), m_locationIdentifierType));
+    query.addQueryItem(u"fromPlace"_s, encodeLocation(req.from()));
+    query.addQueryItem(u"toPlace"_s, encodeLocation(req.to()));
     query.addQueryItem(u"time"_s, req.dateTime().toUTC().toString(Qt::ISODate));
+    query.addQueryItem(u"detailedTransfers"_s, req.includePaths() ? u"true"_s : u"false"_s);
 
     QStringList transitModes, directModes;
     QStringList directModeFormFactors;
@@ -349,7 +366,7 @@ bool Motis2Backend::queryJourney(const JourneyRequest &req, JourneyReply *reply,
 
 bool Motis2Backend::queryTrip(const TripRequest &req, TripReply *reply, QNetworkAccessManager *nam) const
 {
-    const auto tripId = req.journeySection().identifier(m_locationIdentifierType);
+    const auto tripId = req.identifier(m_locationIdentifierType);
     if (tripId.isEmpty()) {
         return false;
     }
